@@ -859,13 +859,20 @@ fr_archive_fake_load (FrArchive *archive)
 
 static void
 load_local_archive (FrArchive  *archive,
-		    const char *uri,
 		    const char *password)
 {
 	FrCommand  *tmp_command;
 	const char *mime_type;
 
-	archive->have_permissions = check_permissions (uri, W_OK);
+	if (! g_file_query_exists (archive->file, NULL)) {
+		fr_archive_action_completed (archive,
+					     FR_ACTION_LOADING_ARCHIVE,
+					     FR_PROC_ERROR_GENERIC,
+					     _("File not found."));
+		return;
+	}
+
+	archive->have_permissions = check_file_permissions (archive->file, W_OK);
 	archive->read_only = ! archive->have_permissions;
 
 	tmp_command = archive->command;
@@ -877,9 +884,10 @@ load_local_archive (FrArchive  *archive,
 			mime_type = get_mime_type_from_magic_numbers (archive->file);
 			if (! create_command_to_load_archive (archive, mime_type)) {
 				archive->command = tmp_command;
+				archive->content_type = mime_type;
 				fr_archive_action_completed (archive,
 							     FR_ACTION_LOADING_ARCHIVE,
-							     FR_PROC_ERROR_GENERIC,
+							     FR_PROC_ERROR_UNSUPPORTED_FORMAT,
 							     _("Archive type not supported."));
 				return;
 			}
@@ -925,7 +933,7 @@ fr_archive_load (FrArchive  *archive,
 		       FR_ACTION_LOADING_ARCHIVE);
 
 	fr_archive_set_uri (archive, uri);
-	load_local_archive (archive, uri, password);
+	load_local_archive (archive, password);
 
 	return TRUE;
 }
@@ -944,7 +952,7 @@ fr_archive_load_local (FrArchive  *archive,
 		       FR_ACTION_LOADING_ARCHIVE);
 
 	fr_archive_set_uri (archive, uri);
-	load_local_archive (archive, uri, password);
+	load_local_archive (archive, password);
 
 	return TRUE;
 }
@@ -1846,8 +1854,12 @@ add_dropped_items (DroppedItemsData *data)
 
 		first_basedir = remove_level_from_path (list->data);
 
-		for (scan = list; scan; scan = scan->next)
-			only_names_list = g_list_prepend (only_names_list, (gpointer) file_name_from_path (scan->data));
+		for (scan = list; scan; scan = scan->next) {
+			char *name;
+
+			name = g_uri_unescape_string (file_name_from_path (scan->data), NULL);
+			only_names_list = g_list_prepend (only_names_list, name);
+		}
 
 		fr_archive_add_files (archive,
 				      only_names_list,
@@ -1859,7 +1871,7 @@ add_dropped_items (DroppedItemsData *data)
 				      data->compression,
 				      data->volume_size);
 
-		g_list_free (only_names_list);
+		path_list_free (only_names_list);
 		g_free (first_basedir);
 
 		return;
@@ -1958,7 +1970,7 @@ fr_archive_add_dropped_items (FrArchive     *archive,
 
 static gboolean
 file_is_in_subfolder_of (const char *filename,
-			 GList      *folder_list)
+		         GList      *folder_list)
 {
 	GList *scan;
 
@@ -1975,13 +1987,10 @@ file_is_in_subfolder_of (const char *filename,
 	return FALSE;
 }
 
-
 static gboolean
 archive_type_has_issues_deleting_non_empty_folders (FrArchive *archive)
 {
-	/*if ((archive->command->files == NULL) || (archive->command->files->len == 0))
-		return FALSE;  FIXME: test with extract_here */
-	return ! archive->command->propCanDeleteNonEmptyFolders;
+       return ! archive->command->propCanDeleteNonEmptyFolders;
 }
 
 
@@ -2118,14 +2127,62 @@ static void
 move_files_to_dir (FrArchive  *archive,
 		   GList      *file_list,
 		   const char *source_dir,
-		   const char *dest_dir)
+		   const char *dest_dir,
+		   gboolean    overwrite)
 {
+	GList *list;
 	GList *scan;
 
+	/* we prefer mv instead of cp for performance reasons,
+	 * but if the destination folder already exists mv
+	 * doesn't work correctly. (bug #590027) */
+
+	list = g_list_copy (file_list);
+	for (scan = list; scan; /* void */) {
+		GList *next = scan->next;
+		char  *filename = scan->data;
+		char  *basename;
+		char  *destname;
+
+		basename = g_path_get_basename (filename);
+		destname = g_build_filename (dest_dir, basename, NULL);
+		if (g_file_test (destname, G_FILE_TEST_IS_DIR)) {
+			fr_process_begin_command (archive->process, "cp");
+			fr_process_add_arg (archive->process, "-R");
+			if (overwrite)
+				fr_process_add_arg (archive->process, "-f");
+			else
+				fr_process_add_arg (archive->process, "-n");
+			if (filename[0] == '/')
+				fr_process_add_arg_concat (archive->process, source_dir, filename, NULL);
+			else
+				fr_process_add_arg_concat (archive->process, source_dir, "/", filename, NULL);
+			fr_process_add_arg (archive->process, dest_dir);
+			fr_process_end_command (archive->process);
+
+			list = g_list_remove_link (list, scan);
+			g_list_free (scan);
+		}
+
+		g_free (destname);
+		g_free (basename);
+
+		scan = next;
+	}
+
+	if (list == NULL)
+		return;
+
+	/* 'list' now contains the files that can be moved without problems */
+
 	fr_process_begin_command (archive->process, "mv");
-	fr_process_add_arg (archive->process, "-f");
-	for (scan = file_list; scan; scan = scan->next) {
+	if (overwrite)
+		fr_process_add_arg (archive->process, "-f");
+	else
+		fr_process_add_arg (archive->process, "-n");
+	for (scan = list; scan; scan = scan->next) {
 		char *filename = scan->data;
+
 		if (filename[0] == '/')
 			fr_process_add_arg_concat (archive->process, source_dir, filename, NULL);
 		else
@@ -2133,6 +2190,8 @@ move_files_to_dir (FrArchive  *archive,
 	}
 	fr_process_add_arg (archive->process, dest_dir);
 	fr_process_end_command (archive->process);
+
+	g_list_free (list);
 }
 
 
@@ -2140,7 +2199,8 @@ static void
 move_files_in_chunks (FrArchive  *archive,
 		      GList      *file_list,
 		      const char *temp_dir,
-		      const char *dest_dir)
+		      const char *dest_dir,
+		      gboolean    overwrite)
 {
 	GList *scan;
 	int    temp_dir_l;
@@ -2164,7 +2224,7 @@ move_files_in_chunks (FrArchive  *archive,
 		}
 
 		prev->next = NULL;
-		move_files_to_dir (archive, chunk_list, temp_dir, dest_dir);
+		move_files_to_dir (archive, chunk_list, temp_dir, dest_dir, overwrite);
 		prev->next = scan;
 	}
 }
@@ -2576,7 +2636,8 @@ fr_archive_extract_to_local (FrArchive  *archive,
 		move_files_in_chunks (archive,
 				      filtered,
 				      temp_dir,
-				      destination);
+				      destination,
+				      overwrite);
 
 		/* remove the temp dir. */
 
