@@ -211,7 +211,7 @@ fr_command_cfile_add (FrCommand     *comm,
 
 	/* copy file to the temp dir */
 
-	temp_dir = get_temp_work_dir ();
+	temp_dir = get_temp_work_dir (NULL);
 	filename = file_list->data;
 	temp_file = g_strconcat (temp_dir, "/", filename, NULL);
 
@@ -249,6 +249,14 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_end_command (comm->process);
 		compressed_filename = g_strconcat (filename, ".Z", NULL);
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzip")) {
+		fr_process_begin_command (comm->process, "lzip");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "--");
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".lz", NULL);
+	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzma")) {
 		fr_process_begin_command (comm->process, "lzma");
 		fr_process_set_working_dir (comm->process, temp_dir);
@@ -257,6 +265,14 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_end_command (comm->process);
 		compressed_filename = g_strconcat (filename, ".lzma", NULL);
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-xz")) {
+		fr_process_begin_command (comm->process, "xz");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "--");
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".xz", NULL);
+	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzop")) {
 		fr_process_begin_command (comm->process, "lzop");
 		fr_process_set_working_dir (comm->process, temp_dir);
@@ -326,7 +342,7 @@ fr_command_cfile_extract (FrCommand  *comm,
 
 	/* copy file to the temp dir, remove the already existing file first */
 
-	temp_dir = get_temp_work_dir ();
+	temp_dir = get_temp_work_dir (NULL);
 	temp_file = g_strconcat (temp_dir,
 				 "/",
 				 file_name_from_path (comm->filename),
@@ -367,6 +383,13 @@ fr_command_cfile_extract (FrCommand  *comm,
 		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzip")) {
+		fr_process_begin_command (comm->process, "lzip");
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, "-d");
+		fr_process_add_arg (comm->process, temp_file);
+		fr_process_end_command (comm->process);
+	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzma")) {
 		fr_process_begin_command (comm->process, "lzma");
 		fr_process_add_arg (comm->process, "-f");
@@ -374,6 +397,13 @@ fr_command_cfile_extract (FrCommand  *comm,
 		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-xz")) {
+		fr_process_begin_command (comm->process, "xz");
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, "-d");
+		fr_process_add_arg (comm->process, temp_file);
+		fr_process_end_command (comm->process);
+	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzop")) {
 		fr_process_begin_command (comm->process, "lzop");
 		fr_process_set_working_dir (comm->process, temp_dir);
@@ -428,9 +458,11 @@ fr_command_cfile_extract (FrCommand  *comm,
 const char *cfile_mime_type[] = { "application/x-gzip",
 				  "application/x-bzip",
 				  "application/x-compress",
+				  "application/x-lzip",
 				  "application/x-lzma",
 				  "application/x-lzop",
 				  "application/x-rzip",
+				  "application/x-xz",
 				  NULL };
 
 
@@ -443,35 +475,44 @@ fr_command_cfile_get_mime_types (FrCommand *comm)
 
 FrCommandCap
 fr_command_cfile_get_capabilities (FrCommand  *comm,
-			           const char *mime_type)
+			           const char *mime_type,
+				   gboolean    check_command)
 {
 	FrCommandCap capabilities;
 
 	capabilities = FR_COMMAND_CAN_DO_NOTHING;
 	if (is_mime_type (mime_type, "application/x-gzip")) {
-		if (is_program_in_path ("gzip"))
+		if (is_program_available ("gzip", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-bzip")) {
-		if (is_program_in_path ("bzip2"))
+		if (is_program_available ("bzip2", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-compress")) {
-		if (is_program_in_path ("compress"))
+		if (is_program_available ("compress", check_command))
 			capabilities |= FR_COMMAND_CAN_WRITE;
-		if (is_program_in_path ("uncompress") || is_program_in_path ("gzip"))
+		if (is_program_available ("uncompress", check_command) || is_program_available ("gzip", check_command))
 			capabilities |= FR_COMMAND_CAN_READ;
 	}
+	else if (is_mime_type (mime_type, "application/x-lzip")) {
+		if (is_program_available ("lzip", check_command))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
 	else if (is_mime_type (mime_type, "application/x-lzma")) {
-		if (is_program_in_path ("lzma"))
+		if (is_program_available ("lzma", check_command))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-xz")) {
+		if (is_program_available ("xz", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-lzop")) {
-		if (is_program_in_path ("lzop"))
+		if (is_program_available ("lzop", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-rzip")) {
-		if (is_program_in_path ("rzip"))
+		if (is_program_available ("rzip", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 
@@ -491,6 +532,31 @@ fr_command_cfile_finalize (GObject *object)
 }
 
 
+static const char *
+fr_command_cfile_get_packages (FrCommand  *comm,
+			       const char *mime_type)
+{
+	if (is_mime_type (mime_type, "application/x-gzip"))
+		return PACKAGES ("gzip");
+	else if (is_mime_type (mime_type, "application/x-bzip"))
+		return PACKAGES ("bzip2");
+	else if (is_mime_type (mime_type, "application/x-compress"))
+		return PACKAGES ("ncompress");
+	else if (is_mime_type (mime_type, "application/x-lzip"))
+		return PACKAGES ("lzip");
+	else if (is_mime_type (mime_type, "application/x-lzma"))
+		return PACKAGES ("lzma");
+	else if (is_mime_type (mime_type, "application/x-xz"))
+		return PACKAGES ("xz");
+	else if (is_mime_type (mime_type, "application/x-lzop"))
+		return PACKAGES ("lzop");
+	else if (is_mime_type (mime_type, "application/x-rzip"))
+		return PACKAGES ("rzip");
+
+	return NULL;
+}
+
+
 static void
 fr_command_cfile_class_init (FrCommandCFileClass *class)
 {
@@ -508,6 +574,7 @@ fr_command_cfile_class_init (FrCommandCFileClass *class)
 	afc->extract          = fr_command_cfile_extract;
 	afc->get_mime_types   = fr_command_cfile_get_mime_types;
 	afc->get_capabilities = fr_command_cfile_get_capabilities;
+	afc->get_packages     = fr_command_cfile_get_packages;
 }
 
 
