@@ -176,7 +176,7 @@ static const char *
 fr_command_jar_get_packages (FrCommand  *comm,
 			     const char *mime_type)
 {
-	return "zip,unzip";
+	return PACKAGES ("zip,unzip");
 }
 
 
