@@ -180,7 +180,8 @@ base_fr_command_get_mime_types (FrCommand *comm)
 
 FrCommandCap
 base_fr_command_get_capabilities (FrCommand  *comm,
-			          const char *mime_type)
+			          const char *mime_type,
+			          gboolean    check_command)
 {
 	return FR_COMMAND_CAN_DO_NOTHING;
 }
@@ -195,6 +196,14 @@ base_fr_command_set_mime_type (FrCommand  *comm,
 }
 
 
+static const char *
+base_fr_command_get_packages (FrCommand  *comm,
+			      const char *mime_type)
+{
+	return NULL;
+}
+
+
 static void
 fr_command_start (FrProcess *process,
 		  gpointer   data)
@@ -215,17 +224,23 @@ fr_command_done (FrProcess   *process,
 	FrCommand *comm = FR_COMMAND (data);
 
 	comm->process->restart = FALSE;
-	if (process->error.type != FR_PROC_ERROR_NONE)
-		fr_command_handle_error (comm, &process->error);
+	fr_command_handle_error (comm, &process->error);
 
-	if (comm->process->restart)
+	if (comm->process->restart) {
 		fr_process_start (comm->process);
-	else
-		g_signal_emit (G_OBJECT (comm),
-			       fr_command_signals[DONE],
-			       0,
-			       comm->action,
-			       &process->error);
+		return;
+	}
+
+	if (comm->action == FR_ACTION_LISTING_CONTENT) {
+		/* order the list by name to speed up search */
+		g_ptr_array_sort (comm->files, file_data_compare_by_path);
+	}
+
+	g_signal_emit (G_OBJECT (comm),
+		       fr_command_signals[DONE],
+		       0,
+		       comm->action,
+		       &process->error);
 }
 
 
@@ -364,6 +379,7 @@ fr_command_class_init (FrCommandClass *class)
 	class->get_mime_types   = base_fr_command_get_mime_types;
 	class->get_capabilities = base_fr_command_get_capabilities;
 	class->set_mime_type    = base_fr_command_set_mime_type;
+	class->get_packages     = base_fr_command_get_packages;
 	class->start            = NULL;
 	class->done             = NULL;
 	class->progress         = NULL;
@@ -567,7 +583,16 @@ fr_command_set_filename (FrCommand  *comm,
 
 
 void
-fr_command_list (FrCommand  *comm)
+fr_command_set_multi_volume (FrCommand  *comm,
+			     const char *filename)
+{
+	comm->multi_volume = TRUE;
+	fr_command_set_filename (comm, filename);
+}
+
+
+void
+fr_command_list (FrCommand *comm)
 {
 	g_return_if_fail (FR_IS_COMMAND (comm));
 
@@ -579,12 +604,19 @@ fr_command_list (FrCommand  *comm)
 	}
 
 	comm->action = FR_ACTION_LISTING_CONTENT;
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process, NULL, NULL);
-	fr_process_set_err_line_func (FR_COMMAND (comm)->process, NULL, NULL);
-	fr_process_use_standard_locale (FR_COMMAND (comm)->process, TRUE);
+	fr_process_set_out_line_func (comm->process, NULL, NULL);
+	fr_process_set_err_line_func (comm->process, NULL, NULL);
+	fr_process_use_standard_locale (comm->process, TRUE);
+	comm->multi_volume = FALSE;
 
-	if (!comm->fake_load)
+	if (! comm->fake_load)
 		FR_COMMAND_GET_CLASS (G_OBJECT (comm))->list (comm);
+	else
+		g_signal_emit (G_OBJECT (comm),
+			       fr_command_signals[DONE],
+			       0,
+			       comm->action,
+			       &comm->process->error);
 }
 
 
@@ -690,15 +722,16 @@ fr_command_get_mime_types (FrCommand *comm)
 void
 fr_command_update_capabilities (FrCommand *comm)
 {
-	comm->capabilities = fr_command_get_capabilities (comm, comm->mime_type);
+	comm->capabilities = fr_command_get_capabilities (comm, comm->mime_type, TRUE);
 }
 
 
 FrCommandCap
 fr_command_get_capabilities (FrCommand  *comm,
-			     const char *mime_type)
+			     const char *mime_type,
+			     gboolean    check_command)
 {
-	return FR_COMMAND_GET_CLASS (G_OBJECT (comm))->get_capabilities (comm, mime_type);
+	return FR_COMMAND_GET_CLASS (G_OBJECT (comm))->get_capabilities (comm, mime_type, check_command);
 }
 
 
@@ -710,6 +743,14 @@ fr_command_is_capable_of (FrCommand     *comm,
 }
 
 
+const char *
+fr_command_get_packages (FrCommand  *comm,
+			 const char *mime_type)
+{
+	return FR_COMMAND_GET_CLASS (G_OBJECT (comm))->get_packages (comm, mime_type);
+}
+
+
 /* fraction == -1 means : I don't known how much time the current operation
  *                        will take, the dialog will display this info pulsing
  *                        the progress bar.
